/*
 * CExpat
 * Copyright (c) 2001 Robert Mooney (rjmooney@aboveground.cx), All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 */

#include "CExpat.h"

// Depth determines # of tabs to be printed
static int depth = -1;

// Display the specified element's information and recurse downward
static void TraverseElementTree( CExpat &expat, element_map_t::iterator element_map_itor, bool bIterateChildren )
{
	// Increment the depth
	++depth;

	// Helper variables
	string element_name = (*element_map_itor).first;
	element_data_t *element_data = & ( (*element_map_itor).second );

	// Determine the element's base name
	string base_name;
	int pos = element_name.rfind( "/", element_name.length() - 1 );
	if ( pos > -1 )  base_name = element_name.substr( pos + 1, element_name.length() - pos - 1 );
	else base_name = element_name;

	// Add tabs before the base tags
	for ( int count = 0;  count < depth;  ++count )  cout << "\t";

	// Display the element path
	cout << "<" << base_name;

	// Attribute map (e.g. name="value", etc)
	element_attribute_map_t *attributes = &( element_data->attributes );

	// Display the element attributes
	for ( element_attribute_map_t::iterator attribute_itor = attributes->begin();
			attribute_itor != attributes->end();
			++attribute_itor
		)
	{
		// Helper variables
		string attribute_name = (*attribute_itor).first;
		string attribute_value = (*attribute_itor).second;

		// The attribute map returns a pair<> when dereferenced
		//
		// pair<>.(first,second) are the attribute (name,value)
		// e.g. name="value"
		//
		cout << " " << attribute_name << "=\"" << attribute_value << "\"";
	}

	// Grab a pointer to the child vector, if it exists
	child_itor_vector_t *child_itor_vector = expat.GetChildItorVector( element_map_itor );

	// Close the element if there are no children to be displayed, and 
	// no data to be enclosed in the element
	//
	if ( !child_itor_vector && element_data->content.length() == 0 )
	{
		cout << "/>" << endl;
		--depth;
		return;
	}

	// Otherwise, just close the tag
	else cout << ">";
			
	// Display the children
	if ( child_itor_vector && bIterateChildren )
	{
		cout << endl;

		// Iterate the children of the parent
		for ( child_itor_vector_t::iterator __itor = child_itor_vector->begin();
				__itor != child_itor_vector->end();
				++__itor )
		{
			element_map_t::iterator element_iterator = *__itor;
			TraverseElementTree( expat, element_iterator, true );
		}

		// Add tabs before the closing tag, since there are children, and like endl's.
		for ( int count = 0;  count < depth;  ++count )  cout << "\t";
	}
			
	// Show the element's value, if available
	if ( element_data->content.length() > 0 )  cout << element_data->content;

	// Close the element
	cout << "</" << base_name << ">" << endl;

	// Decrement the depth
	--depth;
}


// Demo
int main(int argc, char **argv) 
{
	// Open and parse the XML document
	CExpat expat( "test.xml" );

	//
	// Display the requested map elements
	//

	// Have key, search for it and display matches
	if ( argc > 1 )
	{
		// Store the search key, e.g. /TheShell:Config/Session
		string key = argv[1];

		// Grab the range of matching keys
		typedef element_map_t::iterator itor_t;
		pair< itor_t, itor_t > p = expat.m_element_map.equal_range( key );

		// Display the matching entries
		for ( itor_t itor = p.first; itor != p.second; ++itor )  TraverseElementTree( expat, itor, true );

		// Display the number of matching entries
		cout << endl << "Number of matches: " << expat.m_element_map.count( key ) << endl;
	}

	// No key, display all entries
	else
	{
		// Display the tree from the root
		TraverseElementTree( expat, expat.m_element_map.begin(), true );

		// Display the number of matching entries
		cout << endl << "Number of matches: " << expat.m_element_map.size() << endl;
	}

	return 0;
}
